//------------------------------------------------------------------------------
//         Headers
//------------------------------------------------------------------------------

#include <AT91SAM7X256.H> 
#include <lib_at91sam7x256.h>
#include "rtl.h"
#include "gui.h"
#include "ads7843.h"
#include "spi.h"
//------------------------------------------------------------------------------
//         Defines
//------------------------------------------------------------------------------

#define ADS_CTRL_PD0              (1 << 0)        // PD0
#define ADS_CTRL_PD1              (1 << 1)        // PD1
#define ADS_CTRL_DFR              (1 << 2)        // SER/DFR
#define ADS_CTRL_EIGHT_BITS_MOD   (1 << 3)        // Mode
#define ADS_CTRL_START            (1 << 7)        // Start Bit
#define ADS_CTRL_SWITCH_SHIFT     4               // Address setting

// Get X position command
#define CMD_X_POSITION ((1 << ADS_CTRL_SWITCH_SHIFT) | ADS_CTRL_START | ADS_CTRL_PD0 | ADS_CTRL_PD1)
// Get Y position command
#define CMD_Y_POSITION ((5 << ADS_CTRL_SWITCH_SHIFT) | ADS_CTRL_START | ADS_CTRL_PD0 | ADS_CTRL_PD1)
// Enable penIRQ
#define CMD_ENABLE_PENIRQ  ((1 << ADS_CTRL_SWITCH_SHIFT) | ADS_CTRL_START)


#define AT91C_TOUCHSCREEN_TIMEOUT        5000000

#define DELAY_BEFORE_SPCK          (200 << 16) // 2us min (tCSS) <=> 200/100 000 000 = 2us
#define DELAY_BETWEEN_CONS_COM     (0xf << 24) // 5us min (tCSH) <=> (32 * 15) / (100 000 000) = 5us
  
//------------------------------------------------------------------------------
//         Local variables
//------------------------------------------------------------------------------
#define Configure_PIO_BIT(SPI_PORT,SPI_PIN)		SPI_PORT->PIO_PDR=(1<<SPI_PIN);  SPI_PORT->PIO_MDER=(1<<SPI_PIN); SPI_PORT->PIO_ASR=(1<<SPI_PIN);  SPI_PORT->PIO_PPUER=(1<<SPI_PIN) 
#define SPI0_MISO		16
#define SPI0_MOSI		17
#define SPI0_SPCK		18


//------------------------------------------------------------------------------
//         Local functions
//------------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Generic function to send a command to the touchscreen controller
/// /param command to send
/// /return command result
//-----------------------------------------------------------------------------
static unsigned int SendCommand(unsigned char bCmd)
{
   
    unsigned int  uResult = 0;
    unsigned int  uTimeout = 0;
    // (volatile declaration needed for code optimisation by compiler)
    volatile unsigned char bufferRX[3]; 
    volatile unsigned char bufferTX[3];
    
    AT91PS_PDC pPdc = (AT91PS_PDC) &(AT91C_BASE_SPI0->SPI_RPR);
    unsigned int dStatus;  
  
    bufferRX[0] = 0;
    bufferRX[1] = 0;
    bufferRX[2] = 0;
    
    bufferTX[0] = bCmd;
    bufferTX[1] = 0;
    bufferTX[2] = 0;//bCmd;
    
    // Send Command and data through the SPI
    pPdc->PDC_PTCR = AT91C_PDC_RXTDIS;
    pPdc->PDC_RPR = (unsigned int) bufferRX;
    pPdc->PDC_RCR = 3;    
    
    pPdc->PDC_PTCR = AT91C_PDC_TXTDIS;
    pPdc->PDC_TPR = (unsigned int) bufferTX;
    pPdc->PDC_TCR = 3;
    
    pPdc->PDC_PTCR = AT91C_PDC_RXTEN;
    pPdc->PDC_PTCR = AT91C_PDC_TXTEN;

    do {
        dStatus = AT91C_BASE_SPI0->SPI_SR;
        uTimeout++;
    }
    while ((( dStatus & AT91C_SPI_RXBUFF) != AT91C_SPI_RXBUFF) && (uTimeout < AT91C_TOUCHSCREEN_TIMEOUT));

    pPdc->PDC_PTCR = AT91C_PDC_RXTDIS;   
    pPdc->PDC_PTCR = AT91C_PDC_TXTDIS;    
    
    uResult = ((unsigned int)bufferRX[1] << 8) | (unsigned int)bufferRX[2];
    uResult = uResult >> 4; 
    
    return uResult;
	  
}

//------------------------------------------------------------------------------
//         Global functions
//------------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Get position of the pen by ask the ADS controller (SPI)
/// /param pointer to the horizontal position
/// /param pointer to the vertical position
//-----------------------------------------------------------------------------
void ADS7843_GetPosition(unsigned int* px_pos, unsigned int* py_pos)
{
    // Get X position
    *px_pos = SendCommand(CMD_X_POSITION);
    // Get Y position
    *py_pos = SendCommand(CMD_Y_POSITION);
    // Switch to full power mode
    SendCommand(CMD_ENABLE_PENIRQ);	
}


#define READ_TIMES 12
#define LOST_VAL 5	 
void ADS7843_GetPosition2(unsigned int* px_pos, unsigned int* py_pos)
{
	int i, j;
	int bufx[READ_TIMES],bufy[READ_TIMES];
	int sum=0;
	int temp;
	for(i=0;i<READ_TIMES;i++)				 
		ADS7843_GetPosition(bufx+i,bufy+i);	    
				    
	for(i=0;i<READ_TIMES-1; i++)
	{
		for(j=i+1;j<READ_TIMES;j++)
		{
			if(bufx[i]>bufx[j])
			{
				temp=bufx[i];
				bufx[i]=bufx[j];
				bufx[j]=temp;
			}

			if(bufy[i]>bufy[j])
			{
				temp=bufy[i];
				bufy[i]=bufy[j];
				bufy[j]=temp;
			}
		}
	}	  
	sum=0;
	for(i=LOST_VAL;i<READ_TIMES-LOST_VAL;i++)sum+=bufx[i];
	*px_pos=sum/(READ_TIMES-2*LOST_VAL);
	sum=0;
	for(i=LOST_VAL;i<READ_TIMES-LOST_VAL;i++)sum+=bufy[i];
	*py_pos=sum/(READ_TIMES-2*LOST_VAL);
	   
}
//-----------------------------------------------------------------------------
/// Initialization of the SPI for communication with ADS7843 component
//-----------------------------------------------------------------------------
void ADS7843_Initialize(void)
{
    volatile unsigned int uDummy;
      
    // Configure pins
	SPI_Configure_PIO_MAKS(SPI0_PORT,SPI0_PIO_MASK); // Enable the SPI pins required by the application
   
 //   PIO_Configure(pinsSPI, PIO_LISTSIZE(pinsSPI));

    SPI_Configure(AT91C_BASE_SPI0, 
                  AT91C_ID_SPI0,
                  AT91C_SPI_MSTR |
                	AT91C_SPI_MODFDIS |
                 	SPI_PCS(0) // Value of the SPI configuration register.
    );    
    
    SPI_ConfigureNPCS(AT91C_BASE_SPI0, 0, 
                      AT91C_SPI_NCPHA | 
											SPI_SCBR(1000000, 48000000) |
                      SPI_DLYBS(1000, 48000000) | (0xC8 << 8) );
    
    SPI_Enable(AT91C_BASE_SPI0);

    for (uDummy=0; uDummy<100000; uDummy++);

    uDummy = AT91C_BASE_SPI0->SPI_SR;    
    uDummy = AT91C_BASE_SPI0->SPI_RDR;  
    
    SendCommand(CMD_ENABLE_PENIRQ);    
	
}

//-----------------------------------------------------------------------------
/// Reset the ADS7843
//-----------------------------------------------------------------------------
void ADS7843_Reset(void)
{
    // Disable SPI 0
  //  SPI_Disable(AT91C_BASE_SPI0);
}

